/*
** Copyright (c) 2003 CA.  All rights reserved.  
** This software may not be duplicated, disclosed or reproduced in whole or 
** in part for any purpose except as authorized by the applicable license agreement, 
** without the express written authorization of CA. All authorized reproductions 
** must be marked with this language.
**
** TO THE EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS 
** SOFTWARE WITHOUT WARRANTY OF ANY KIND, INCLUDING 
** WITHOUT LIMITATION, ANY IMPLIED WARRANTIES OF MERCHANTABILITY, 
** FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT.  IN NO EVENT 
** WILL CA BE LIABLE TO THE END USER OR ANY THIRD PARTY FOR ANY LOSS 
** OR DAMAGE, DIRECT OR INDIRECT, FROM THE USE OF THIS MATERIAL, 
** INCLUDING WITHOUT LIMITATION, LOST PROFITS, BUSINESS 
** INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS EXPRESSLY 
** ADVISED OF SUCH LOSS OR DAMAGE.
*/

// SiteMinder Java Authentication API sample.

package com.netegrity.sdk.javaauthapi;

import java.util.Date;
import java.text.DateFormat;
import com.netegrity.policyserver.smapi.*;

import javax.security.cert.X509Certificate;
import javax.security.cert.CertificateException;
import java.util.regex.*;
/**
 * A simple SiteMinder Java Authentication Scheme.
 */

public class AuthApiCertSample implements SmAuthScheme  {
    private static final int SCHEME_VERSION = SmAuthQueryResponse.SMAUTH_API_VERSION_V3;
    private static final String SCHEME_DESCRIPTION = "SiteMinder(tm) Example Java authentication scheme, (c) Netegrity 2003";


    /**
     * Returns information about the authentication scheme.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @param request - The request code, SMAUTH_QUERY_DESCRIPTION or SMAUTH_QUERY_CREDENTIALS_REQ.
     * @param response - Holds methods by which query() returns the requested information.
     * @return: If successful returns SMAUTH_SUCCESS, otherwise returns SMAUTH_FAILURE
     */

    public SmAuthStatus query(String parameter, String secret, SmAuthQueryCode request, SmAuthQueryResponse response) {

        log(" query : " + parameter + "," + secret + ", " + request + ", " + response);

        if (null == response) {
            return SmAuthStatus.SMAUTH_FAILURE;
        }

        if (SmAuthQueryCode.SMAUTH_QUERY_DESCRIPTION == request) {
            response.setResponseBuffer(SCHEME_DESCRIPTION);
            response.setResponseCode(SCHEME_VERSION);
        }
        else if (SmAuthQueryCode.SMAUTH_QUERY_CREDENTIALS_REQ == request) {
            //response.setResponseCode(SmAuthQueryResponse.SMAUTH_CRED_BASIC);
            //response.setResponseCode(SmAuthQueryResponse.SMAUTH_CRED_FORM_REQUIRED);
            response.setResponseCode(SmAuthQueryResponse.SMAUTH_CRED_SSL_REQUIRED|SmAuthQueryResponse.SMAUTH_CRED_X509CERT|SmAuthQueryResponse.SMAUTH_CRED_X509CERT_USER_DN);
            //response.setResponseCode(SmAuthQueryResponse.SMAUTH_CRED_X509CERT);
            response.setResponseBuffer(parameter);
            System.out.println(" query::Parameter : " + parameter);
            System.out.println(" query::ParameterX : " + this.parameter);
            //response.setResponseCode(SmAuthQueryResponse.SMAUTH_CRED_X509CERT | SmAuthQueryResponse.SMAUTH_CRED_X509CERT_USER_DN);

        }
        else {
            return SmAuthStatus.SMAUTH_FAILURE;
        }

        return SmAuthStatus.SMAUTH_SUCCESS;
    }


    /**
     * SiteMinder invokes this method so the authentication scheme can
     * perform its own initialization procedure. This method is invoked once
     * for each authentication scheme instance when it is first loaded.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @return: If successful returns SMAUTH_SUCCESS, otherwise returns SMAUTH_FAILURE
     */

    String parameter = null;

    public SmAuthStatus  init(String parameter, String secret)  {

        this.parameter = parameter;

        log(" init::Parameter : " + parameter + " " + secret);

        return SmAuthStatus.SMAUTH_SUCCESS;
    }


    /**
     * SiteMinder invokes this method during shutdown so the authentication
     * scheme can perform its own rundown procedure. This method is invoked
     * once for each authentication scheme instance during SiteMinder shutdown.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @return: If successful returns SMAUTH_SUCCESS, otherwise returns SMAUTH_FAILURE
     */

    public SmAuthStatus release(String parameter, String secret) {
        return SmAuthStatus.SMAUTH_SUCCESS;
    }


    void log(String msg) {
         System.out.println(getClass().getSimpleName() + " AuthApiCert:: ['" + msg +"']");
    }

    void log(SmAuthenticationContext context, String msg) {
         context.getAPIContext().trace(getClass().getSimpleName(), "AuthApiCert:: ['" + msg +"']");
//         System.out.println(this.getClass().getName() + " AuthApiCert:: ['" + msg +"']");
         System.out.println(getClass().getSimpleName() + " AuthApiCert:: ['" + msg +"']");
    }

    /**
     * SiteMinder invokes this method to authenticate user credentials.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @param challengeReason - The reason for the original authentication challenge, or 0 if unknown.
     * @param context - Contains request context and methods to return message buffers.
     *
     * @return: an SmAuthenticationResult object.
     */


    public SmAuthenticationResult authenticate(String parameter, String secret, int challengeReason,  SmAuthenticationContext context) {
        // cannot do authentication without the authentication context

        log(context, " authenticate::Parameter : " + parameter + ", " + secret + ", " + challengeReason + ", " + context);


        if (null == context) {
            return new SmAuthenticationResult(SmAuthStatus.SMAUTH_NO_USER_CONTEXT, SmAuthenticationResult.REASON_NONE);
        }

    	log(context, " " + context.getAPIContext().getVersion() +" ");

        UserCredentialsContext userCredentialsContext = context.getUserCredentialsContext();

        if ( userCredentialsContext == null) {
                log(context, "user Credentials are null");
        	return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }

        UserContext theUserContext = context.getUserContext();

        /*
        // If the scheme is designed not to disambiguate users, it should return SmAuthApi_NoUserContext.
        if ((null == theUserContext) || !theUserContext.isUserContext()) {
            //return  new SmAuthenticationResult(SmAuthStatus.SMAUTH_NO_USER_CONTEXT, SmAuthenticationResult.REASON_NONE);
        }        
        */

        // Resolving context using the certificate details.
        if ((null == theUserContext) || !theUserContext.isUserContext()) {

                byte[] rawCertificate = userCredentialsContext.getCertAsBinary();
                log(context, "rawCertificate len: " + rawCertificate.length);

                String dn = userCredentialsContext.getCertUserDN();
                log(context, "raw DN '" + dn + "' (this does not seem to be collected)" );

                if (rawCertificate == null) {
                  log(context, "Unable to retrieve certificate from context");
                  return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
                }
            
                X509Certificate userX509Cert;
                try {
                    userX509Cert = X509Certificate.getInstance(rawCertificate);
                }
                catch (CertificateException ce) {
                   log(context, "Certificate Exception : " + ce.getMessage());
                   return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
                }

                String userDN = userX509Cert.getSubjectDN().getName();
                log(context, "User DN : " + userDN);

                
                /*
                // If using complete DN then use the following.
                  context.setUserText(userDN);
                   return new SmAuthenticationResult(SmAuthStatus.SMAUTH_SUCCESS_USER_DN, SmAuthenticationResult.REASON_NONE);
                */

                // Else we parse the user name from the DN and use that as the user.
                Pattern patternCN = Pattern.compile("CN=([^,]*)");
                Matcher matcher = patternCN.matcher(userDN);

                if (!matcher.find()) { // Did not find CN=  in the distinguished name,
                   return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
                }

                String userName = matcher.group(1);
                log(context, "UserName : " + userName);

                context.setUserText(userName);
                return new SmAuthenticationResult(SmAuthStatus.SMAUTH_SUCCESS, SmAuthenticationResult.REASON_NONE);

        }

        // Authentication

        // Generally at this point, the use is authenticated, since the certifcate was accepted by the webserver so it 
        // passed the CRL, and date validation etc.  We can also do some additional checks here as well.

        // This would need a change in spot where the userX509Cert was loaded, since we just load it in disabiguate phase above
         /*
        try { 
            userX509Cert.checkValidity();
        }
        catch (CertificateExpiredException e) {
            return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }
        catch (CertificateNotYetValidException e) {
            return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }
        */

        // Check if the user account is disabled.
        try {
            if (0 != Integer.parseInt(theUserContext.getProp("disabled")))  {
                context.setUserText("User account is disabled.");

                return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_USER_DISABLED);
            }
        }
        catch (NumberFormatException exc) {
            // Do nothing -- the user is not disabled
        }

        // Ok we passed all that, so we are authenticated, note we did not call the password authenticate module, since we 
        // did not need to check the password, the fact this user existed in the user store, and had a certificat that matches
        // is enough.

        return new SmAuthenticationResult(SmAuthStatus.SMAUTH_ACCEPT, SmAuthenticationResult.REASON_NONE);
    }
}

// EOF AuthApiSample.java
